import React from "react";
import cockpit from "cockpit";
import {
    CodeBlock,
    CodeBlockCode,
    Page,
    PageSection,
    PageSectionVariants,
    Stack
} from "@patternfly/react-core";
import { ExclamationCircleIcon, SpinnerIcon } from "@patternfly/react-icons";
import { EmptyStatePanel } from "../../pkg/lib/cockpit-components-empty-state.jsx";
import { HelperText, HelperTextItem } from "@patternfly/react-core/dist/esm/components/HelperText";

const _ = cockpit.gettext;

export function ErrorIcon () {
    return (
        <HelperText>
            <HelperTextItem variant='error' hasIcon>
                {_("Error")}
            </HelperTextItem>
        </HelperText>
    );
}

export const ErrorMessage = ({
    loading,
    error
}) => {
    return (
        <Page>
            <PageSection variant={PageSectionVariants.light}>
                <EmptyStatePanel
                    icon={error ? ExclamationCircleIcon : SpinnerIcon}
                    loading={loading}
                    title={error ? _("Something went wrong") : _("Loading. Please wait...")}
                    paragraph={
                        error && (
                            <Stack hasGutter>
                                <div>{_("There was an unexpected error.")}</div>
                                {error === "access-denied" &&
                                <div>{_("Error: Root permissions required for this page.")}</div>}
                                <CodeBlock>
                                    <CodeBlockCode
                                        id='code-content'
                                    >{error}
                                    </CodeBlockCode>
                                </CodeBlock>
                            </Stack>)
                    }
                />
            </PageSection>
        </Page>
    );
};
